using CoreLibrary;
using CoreLibrary.Regles;
using MauiSpark.Pages;

namespace MauiSpark.Vues;

/// <summary>
/// Vue permettant de slectionner un mode de jeu.
/// </summary>
public partial class ModeVue : ContentView
{
    /// <summary>
    /// Proprit de dpendance pour les rgles du jeu.
    /// </summary>
    public static readonly BindableProperty ReglesProperty = BindableProperty.Create(nameof(Regles), typeof(IRegles), typeof(ModeVue), null);

    /// <summary>
    /// Les rgles du jeu slectionnes.
    /// </summary>
    public IRegles Regles
    {
        get => (IRegles)GetValue(ReglesProperty);
        set => SetValue(ReglesProperty, value);
    }

    /// <summary>
    /// Constructeur par dfaut de la vue du mode de jeu.
    /// </summary>
    public ModeVue()
	{
		InitializeComponent();
	}

    /// <summary>
    /// Mthode dclenche lorsque le bouton de slection des rgles est press.
    /// </summary>
    /// <param name="sender">L'objet qui a dclench l'vnement ; ici, le bouton de slection des rgles.</param>
    /// <param name="e">Les arguments de l'vnement EventArgs.</param>
    private void QuandReglesPresse(Object sender, EventArgs e)
    {
        Partie partie = MauiProgram.Manageur.NouvellePartie(Regles);

        partie.PartieDemanderJoueur += new ConnexionPage().QuandDemanderNom;
        partie.PartiePartieTerminee += new VictoirePage().QuandPartieTerminee;
        partie.PartieNouveauTour += new PlateauPage().QuandNouveauTour;

        partie.Jouer();
    }
}